NSB = LibStub("AceAddon-3.0"):NewAddon("NugStackBar","AceConsole-3.0","AceEvent-3.0")

NugStackBar_FADE_IN = 0.3;
NugStackBar_FADE_OUT = 0.5;
NugStackBar_HIGHLIGHT_FADE_IN = 0.4;
NugStackBar_SHINE_FADE_IN = 0.3;
NugStackBar_SHINE_FADE_OUT = 0.4;
NugStackBar_FRAME_LAST_NUM_POINTS = 0;

local showtypes = {
	["always"]          = "Always show",
	["combat"]          = "Only in combat",
	["normal"]          = "Normal",
}

local skins = {
    ["oldstyle"]        = "NSB Classic",
	["striped"]         = "Striped",
}

local dbDefaults = {
    profile = {
        posX = 0,
        posY = 0,
        scale = 1.0,
        skin = "oldstyle",
        showopts = "normal",
        colors = {
            [1] = {0.3,0.2,1},
            [2] = {0.3,0.2,1},
            [3] = {0.3,0.2,1},
            [4] = {0.3,0.2,1},
            [5] = {0.3,0.2,1},
            [6] = {1,1,1},
        }
    }
}

function NSB:OnInitialize()

    self.db = LibStub:GetLibrary("AceDB-3.0"):New("NugStackBarDB", dbDefaults, "Default")
    self.combat = 0
    self.showBg = 0
    self.debuffName = nil
    self.opt = {
		type = 'group',
        name = "NugStackBar",
        args = {},
	}
    self.opt.args.display = {
        type    = "group",
        name    = "Display Settings",
        order   = 1,
        args    = {},
    }
    self.opt.args.display.args.general = {
        type = "group",
        name = "General",
        order = 1,
        args = {
            showPositon = {
                type = "group",
                name = "Position",
                guiInline = true,
                order = 1,
                args = {
                    posX = {
                        name = "Pos X",
                        type = "range",
                        desc = "Horizontal position, relative to center",
                        get = function(info) return self.db.profile.posX end,
                        set = function(info, s) self.db.profile.posX = s; self.frame:SetPoint("CENTER",UIParent,"CENTER",self.db.profile.posX,self.db.profile.posY); end,
                        min = -900,
                        max = 900,
                        step = 5,
                    },
                    posY = {
                        name = "Pos Y",
                        type = "range",
                        desc = "Vertical position, relative to center",
                        get = function(info) return self.db.profile.posY end,
                        set = function(info, s) self.db.profile.posY = s; self.frame:SetPoint("CENTER",UIParent,"CENTER",self.db.profile.posX,self.db.profile.posY); end,
                        min = -700,
                        max = 700,
                        step = 5,
                    },
                },
            },
            showGeneral = {
                type = "group",
                name = "General",
                guiInline = true,
                order = 2,
                args = {
                    scale = {
                        name = "Scale",
                        type = "range",
                        desc = "Change scale",
                        get = function(info) return self.db.profile.scale end,
                        set = function(info, s) self.db.profile.scale = s; self.frame:SetScale(self.db.profile.scale); end,
                        min = 0.4,
                        max = 2,
                        step = 0.2,
                    },
                    showopts = {
                        type = "select",
                        name = "Show",
                        desc = "When...",
                        values = showtypes,
                        get = function(info)
                            return self.db.profile.showopts 
                        end,
                        set = function(info, s)
                            self.db.profile.showopts = s
                            NSB:UpdateShowOpts()
                        end,
                    },
                }
            },
            showColor = {
                type = "group",
                name = "Colors",
                guiInline = true,
                order = 3,
                args = {
                    color1 = {
                        name = "1st",
                        type = 'color',
                        desc = "Color of first point",
                        get = function(info)
                            local r,g,b = unpack(self.db.profile.colors[1])
                            return r,g,b
                        end,
                        set = function(info, r, g, b)
                            self:SetColor(1,r,g,b)
                        end,
                    },
                    color2 = {
                        name = "2nd",
                        type = 'color',
                        desc = "Color of second point",
                        get = function(info)
                            local r,g,b = unpack(self.db.profile.colors[2])
                            return r,g,b
                        end,
                        set = function(info, r, g, b)
                            self:SetColor(2,r,g,b)
                        end,
                    },
                    color3 = {
                        name = "3rd",
                        type = 'color',
                        desc = "Color of third point",
                        get = function(info)
                            local r,g,b = unpack(self.db.profile.colors[3])
                            return r,g,b
                        end,
                        set = function(info, r, g, b)
                            self:SetColor(3,r,g,b)
                        end,
                    },
                    color4 = {
                        name = "4th",
                        type = 'color',
                        desc = "Color of fourth point",
                        get = function(info)
                            local r,g,b = unpack(self.db.profile.colors[4])
                            return r,g,b
                        end,
                        set = function(info, r, g, b)
                            self:SetColor(4,r,g,b)
                        end,
                    },
                    color5 = {
                        name = "5th",
                        type = 'color',
                        desc = "Color of fifth point",
                        get = function(info)
                            local r,g,b = unpack(self.db.profile.colors[5])
                            return r,g,b
                        end,
                        set = function(info, r, g, b)
                            self:SetColor(5,r,g,b)
                        end,
                    },
                    color = {
                        name = "ALL points",
                        type = 'color',
                        desc = "Color of all points",
                        get = function(info)
                            local r,g,b = unpack(self.db.profile.colors[1])
                            return r,g,b
                        end,
                        set = function(info, r, g, b)
                            for i=1,5 do
                                self:SetColor(i,r,g,b)
                            end
                        end,
                    },
                    colorBG = {
                        name = "Background",
                        type = 'color',
                        desc = "Background color",
                        get = function(info)
                            local r,g,b = unpack(self.db.profile.colors[6])
                            return r,g,b
                        end,
                        set = function(info, r, g, b)
                            self:SetColor(6,r,g,b)
                        end,
                    },
                },
            },
        },
    }
    self.opt.args.display.args.skin = {
        type = "group",
        name = "Skin",
        order = 2,
        args = {
            skin = {
                order = 1,
                type = "select",
                name = "Skin (requires ReloadUI)",
                desc = "Choose one",
                values = skins,
                get = function(info)
                    return self.db.profile.skin
                end,
                set = function(info, s)
                    self.db.profile.skin = s
                end,
            },
            reloadui = {
                order = 2,
                type = "execute",
                name = "ReloadUI",
                desc = "Reloads your UI",
                func = function() ReloadUI() end
            },
        },
    }
    
    LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("NugStackBar", self.opt)
    self:RegisterChatCommand("nstb",function() LibStub("AceConfigDialog-3.0"):Open("NugStackBar") end)
    self:RegisterChatCommand("nugstackbar",function() LibStub("AceConfigDialog-3.0"):Open("NugStackBar") end)
--~     LibStub("AceConfigCmd-3.0"):HandleCommand("nsb","NugStackBar","")
    self.frame = NSBCreateFrame(self.db.profile.skin)
--~     self.tex = self:CreateTex()
    self:UpdateShowOpts()
    
--~     self.timer = LibStub("AceTimer-3.0"):ScheduleRepeatingTimer(TimerTest, 0.3, self.tex)
end

function NSB:OnEnable()
    if (self:SetupDebuff()) then
        self:RegisterEvent("PLAYER_TARGET_CHANGED", "UpdateDebuffs");
        self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
    end
end

function NSB:OnDisable()
    self:UnregisterEvent("PLAYER_TARGET_CHANGED");
    self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
    self.frame:Hide()
end

function NSB:COMBAT_LOG_EVENT_UNFILTERED(event, timestamp, eventType, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, spellID, spellName, spellSchool, auraType)
    
    local isDestTarget = (bit.band(dstFlags, COMBATLOG_OBJECT_TARGET) == COMBATLOG_OBJECT_TARGET)
    
    if  (  eventType == "SPELL_AURA_APPLIED"
        or eventType == "SPELL_AURA_APPLIED_DOSE"
        or eventType == "SPELL_AURA_REMOVED_DOSE"
        or eventType == "SPELL_AURA_REMOVED"
        or eventType == "SPELL_AURA_DISPELLED" )
        and isDestTarget and auraType == "DEBUFF" then
		if string.find(spellName,self.debuffName) ~= nil then
            self:UpdateDebuffs()
        end
    end
end

function NSB:TalentInfo(talentname)
local numTabs = GetNumTalentTabs();
for t=1, numTabs do
    local numTalents = GetNumTalents(t);
    for i=1, numTalents do
        nameTalent, icon, tier, column, currRank, maxRank= GetTalentInfo(t,i);
        if (nameTalent == talentname) then return currRank; end
    end
end
return 0;
end


function NSB:SetColor(point, r, g, b)
if point < 6 then
    self.db.profile.colors[point] = {r,g,b}
    getglobal("NugStackBarPoint"..point.."Highlight"):SetVertexColor(r,g,b);
else
    self.db.profile.colors[6] = {r,g,b}
    getglobal("NugStackBarBackground"):SetVertexColor(r,g,b);
end
end

function NSB:UpdateDebuffs()
    local comboPoints = self:GetDebuffStacks();
	local comboPoint, comboPointHighlight, fadeInfo;
    if ( comboPoints > 0 - self.showBg - self.combat) then
		if ( not NugStackBar:IsVisible() ) then
			NugStackBar:Show();
			UIFrameFadeIn(NugStackBar, NugStackBar_FADE_IN);
		end

		for i=1, MAX_COMBO_POINTS do
			comboPointHighlight = getglobal("NugStackBarPoint"..i.."Highlight");
			comboPointShine = getglobal("NugStackBarPoint"..i.."Shine");
			if ( i <= comboPoints ) then
				if ( i > NugStackBar_FRAME_LAST_NUM_POINTS ) then
					fadeInfo = {};
					fadeInfo.mode = "IN";
					fadeInfo.timeToFade = NugStackBar_HIGHLIGHT_FADE_IN;
					fadeInfo.finishedFunc = function(frame) self:ShineFadeIn(frame) end;
					fadeInfo.finishedArg1 = comboPointShine;
					UIFrameFade(comboPointHighlight, fadeInfo);
				end
			else
				comboPointHighlight:SetAlpha(0);
				comboPointShine:SetAlpha(0);
			end
		end
	else
		NugStackBarPoint1Highlight:SetAlpha(0);
		NugStackBarPoint1Shine:SetAlpha(0);
		NugStackBar:Hide();
	end
	NugStackBar_FRAME_LAST_NUM_POINTS = comboPoints;
end

function NSB:PLAYER_REGEN_ENABLED()
    self.combat = 0
    self.frame:Hide()
end

function NSB:PLAYER_REGEN_DISABLED()
    self.combat = 1
    self.frame:Show()
end

function NSB:UpdateShowOpts()
    if self.db.profile.showopts == "combat" then
        self:RegisterEvent("PLAYER_REGEN_ENABLED")
        self:RegisterEvent("PLAYER_REGEN_DISABLED")
    else
        self:UnregisterEvent("PLAYER_REGEN_ENABLED")
        self:UnregisterEvent("PLAYER_REGEN_DISABLED")
        self.combat = 0
    end
    if self.db.profile.showopts == "always" then
        self.showBg = 1
        self.frame:Show()
    else
        self.showBg = 0
    end
end

function NSB:ShineFadeIn(frame)
	local fadeInfo = {};
	fadeInfo.mode = "IN";
	fadeInfo.timeToFade = NugStackBar_SHINE_FADE_IN;
	fadeInfo.finishedFunc = function(frameName) self:ShineFadeOut(frameName) end;
	fadeInfo.finishedArg1 = frame:GetName();
	UIFrameFade(frame, fadeInfo);
end

function NSB:ShineFadeOut(frameName)
	UIFrameFadeOut(getglobal(frameName), NugStackBar_SHINE_FADE_OUT);
end

function NSB:GetDebuffStacks()
    if ( UnitName("target") == nil )then return 0,0; end
    local stacks = 0;
        for i = 1,40  do
            name, _, _, count, _, duration, _ = UnitDebuff("target", i)
    --~         name, rank, icon, count, debuffType, duration, timeLeft = UnitDebuff("target", i)
            if ( name == nil) then break; end;
            if ( string.find(name,self.debuffName) ~= nil) then
                if ( class == "ROGUE" or class == "DRUID" or class == "PALADIN") then
                    if ( duration ~= nil ) then
                        stacks = count;
                        break
                    end
                else
                    stacks = count;
                    break
                end
            end
        end
    return stacks, rank;
end

function NSB:SetupDebuff()
    _, class = UnitClass("player");
    if (class == "WARRIOR") then self.debuffName="Sunder Armor"; end
    if (class == "PALADIN") then self.debuffName="Holy Vengeance"; end
    if (class == "DRUID") then self.debuffName="Lacerate"; end
    if (class == "ROGUE") then self.debuffName="Deadly Poison"; this:RegisterEvent("CHAT_MSG_SPELL_SELF_DAMAGE"); end
    if (class == "PRIEST") then
        if self:TalentInfo("Shadow Weaving") > 0 then self.debuffName="Shadow Vulnerability"; end
    end
    if (class == "MAGE") then
        if self:TalentInfo("Improved Scorch") > 0 then self.debuffName="Fire Vulnerability";
        elseif self:TalentInfo("Winter's Chill") > 0 then self.debuffName="Winter's Chill"; end
    end
    return self.debuffName
end
